/*
 Copyright 2015 Nicolas Melot

 This file is part of Static-Dynamic-Busy.

 Static-Dynamic-Busy is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Static-Dynamic-Busy is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Static-Dynamic-Busy. If not, see <http://www.gnu.org/licenses/>.

*/


#include <iostream>
#include <cstdlib>
#include <fstream>
#include <string>
#include <boost/regex.hpp>
#include <iomanip>

#include <pelib/GraphML.hpp>
#include <pelib/XMLSchedule.hpp>
#include <pelib/AmplInput.hpp>
#include <pelib/AmplOutput.hpp>

#include <pelib/AmplOutputScalar.hpp>
#include <pelib/AmplOutputVector.hpp>
#include <pelib/AmplOutputSet.hpp>
#include <pelib/AmplOutputMatrix.hpp>

#include <pelib/AmplInputScalar.hpp>
#include <pelib/AmplInputVector.hpp>
#include <pelib/AmplInputSet.hpp>
#include <pelib/AmplInputMatrix.hpp>

#ifndef debug
#if 1
#define debug(var) cout << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "] " << #var << " = \"" << var << "\"" << endl;
#else
#define debug(var)
#endif
#endif

using namespace pelib;
using namespace std;

/*
const float alpha = 3.0;
const float epsilon = 0.5;
*/

typedef struct
{
	float alpha, epsilon, kappa, eta, zeta;
	char *taskgraph, *platform, *schedule;
	bool describe;
} args_t;

static args_t
parse_arguments(char **argv)
{
	args_t args;

	// default values
	args.alpha = 3.0;
	args.epsilon = 0.5;
	args.kappa = 1;
	args.eta = 0.1;
	args.describe = false;

	for(argv++; argv[0] != NULL; argv++)
	{
		if(string(argv[0]).compare("--alpha") == 0)
		{
			argv++;
			stringstream ss(argv[0]);
			ss >> args.alpha;
			continue;
		}
		if(string(argv[0]).compare("--epsilon") == 0)
		{
			argv++;
			stringstream ss(argv[0]);
			ss >> args.epsilon;
			continue;
		}
		if(string(argv[0]).compare("--kappa") == 0)
		{
			argv++;
			stringstream ss(argv[0]);
			ss >> args.kappa;
			continue;
		}
		if(string(argv[0]).compare("--eta") == 0)
		{
			argv++;
			stringstream ss(argv[0]);
			ss >> args.eta;
			continue;
		}
		if(string(argv[0]).compare("--taskgraph") == 0)
		{
			argv++;
			if(argv[0][0] == '-') { argv--;	continue; }
			args.taskgraph = argv[0];
			continue;
		}
		if(string(argv[0]).compare("--platform") == 0)
		{
			argv++;
			if(argv[0][0] == '-') { argv--;	continue; }
			args.platform = argv[0];
			continue;
		}
		if(string(argv[0]).compare("--schedule") == 0)
		{
			argv++;
			if(argv[0][0] == '-') { argv--;	continue; }
			args.schedule = argv[0];
			continue;
		}
		if(string(argv[0]).compare("--label") == 0)
		{
			args.describe = 1;
		}
	}

	return args;
}

int
main(int argc, char **argv)
{
	args_t args = parse_arguments(argv);
	if(args.describe)
	{
		// Just output a description string
		cout << "Stat./dyn.a=" << args.alpha << ",ep=" << args.epsilon << ",k=" << args.kappa << ",et=" << args.eta << endl;
	}
	else
	{
		AmplOutput output(AmplOutput::floatHandlers());

		cout << setprecision(6)
		<< setiosflags(ios::fixed)
		<< setiosflags(ios::showpoint);

		std::ifstream tgStream;
		tgStream.open (args.taskgraph, std::ios::in);
		std::ifstream archStream;
		archStream.open (args.platform, std::ios::in);
		std::ifstream schedStream;
		schedStream.open (args.schedule, std::ios::in);
		Algebra rec = AmplInput(AmplInput::floatHandlers()).parse(archStream);
		Platform plat(rec);
		Taskgraph *tg = GraphML().parse(tgStream);
		rec = rec.merge(tg->buildAlgebra(plat));
		delete tg;
		rec = rec.merge(XMLSchedule().parse(schedStream)->buildAlgebra());
		tgStream.close();
		archStream.close();
		schedStream.close();

		int n = rec.find<Scalar<float> >("n")->getValue();
		int p = rec.find<Scalar<float> >("p")->getValue();
		float M = rec.find<Scalar<float> >("M")->getValue();
		Vector<int, float> tau = rec.find<Vector<int, float> >("Tau");
		Vector<int, float> Wi = rec.find<Vector<int, float> >("Wi");
		Vector<int, float> frequency = rec.find<Vector<int, float> >("frequency");
		Set<float> F = rec.find<Set<float> >("F");
		
		Matrix<int, int, float> e = rec.find<Matrix<int, int, float> >("e");
		Matrix<int, int, float> schedule = rec.find<Matrix<int, int, float> >("schedule");
		vector<int> width(n, 0);
		
		double idle_energy = p * M * pow(args.kappa, args.alpha);
		double busy_energy = 0;

		//AmplOutput(AmplOutput::floatHandlers()).dump(cerr, schedule);

		// First get the width of each task
		for(map<int, map<int, float> >::const_iterator i = schedule.getValues().begin(); i != schedule.getValues().end(); i++)
		{
			//int core = i->first;
			for(map<int, float>::const_iterator j = i->second.begin(); j != i->second.end(); j++)
			{
				int task = j->second;
				if(task > 0)
				{
					//cerr << "Core " << core << ", task " << task << ", width = " << width[task - 1] << endl;
					float width_t = width[task - 1];
					width_t++;
					width[task - 1] = width_t;
				}
			}
		}

		// Now evaluate the quality of this schedule
		float total_area = 0;
		//float max_end_time = 0;
		for(map<int, map<int, float> >::const_iterator i = schedule.getValues().begin(); i != schedule.getValues().end(); i++)
		{
			//int core = i->first;
			//float end_time = 0;
			for(map<int, float>::const_iterator j = i->second.begin(); j != i->second.end(); j++)
			{
				int task = j->second;
				//cerr << "Core " << core << ", task " << task << endl;
				if(task > 0)
				{
					float tau_t = tau.find(task);
					float width_t = width[task - 1];
					double e_wt = e.find(width_t, task);
					double f_t = (double)frequency.find(task);
					double time = (double)tau_t / (e_wt * f_t);
					total_area += time;
					//end_time += area;
					busy_energy += time * (args.epsilon * pow(frequency.find(task), args.alpha) + (1 - args.epsilon) * (frequency.find(task) + args.kappa));

					//cerr << "Task " << task << ", tau " << tau_t << ", width " << width_t << ", e " << e_wt << ", area " << area << ", f " << f_t << ", energy " << area * pow(frequency.find(task), args.alpha) << endl;
				}
			}

			/*	
			if(end_time > max_end_time)
			{
				max_end_time = end_time;
			}
			*/
		}
		idle_energy = (M * p - total_area) * args.eta;

		/*
		if(max_end_time > M)
		{
			cerr << "Invalid schedule because of violating makespan (" << max_end_time << " > " << M << ")." << endl;
		}
		cout << "end_time = " << max_end_time << endl;
		*/

		typedef std::map<int, float> RowType;
		typedef std::map<int, RowType> MatrixType;

		RowType xs, ys;
		xs[0] = 0;
		xs[1] = 1;
		ys[0] = busy_energy + (1 - args.epsilon) * idle_energy;
		ys[1] = ys[0];

		MatrixType matrix;
		matrix[0] = xs;
		matrix[1] = ys;

		AmplOutput out(AmplOutput::floatHandlers());
		Matrix<int, int, float> energy("time__power", matrix);
		Matrix<int, int, float> energy_transposed = energy.transpose();
		out.dump(cout, &energy_transposed);
		//cout << dynamic_energy + args.epsilon * static_energy << " " << rec.find<Scalar<float> >("time")->getValue() << endl;

		if(isnan(ys[0]) || isinf(ys[0]))
		{
			return EXIT_FAILURE;
		}
	}

	return EXIT_SUCCESS;
}

